/**
 * @file spectrum.hpp
 */
/* OpenCMISS-Zinc Library
*
* This Source Code Form is subject to the terms of the Mozilla Public
* License, v. 2.0. If a copy of the MPL was not distributed with this
* file, You can obtain one at http://mozilla.org/MPL/2.0/. */
#ifndef CMZN_SPECTRUM_HPP__
#define CMZN_SPECTRUM_HPP__

#include "zinc/spectrum.h"
#include "zinc/context.hpp"

namespace OpenCMISS
{
namespace Zinc
{

class Spectrumcomponent
{
protected:
    cmzn_spectrumcomponent_id id;

public:

    Spectrumcomponent() :
        id(0)
    {
    }

    // takes ownership of C handle, responsibility for destroying it
    explicit Spectrumcomponent(
        cmzn_spectrumcomponent_id in_spectrumcomponent_id) :
        id(in_spectrumcomponent_id)
    {
    }

    Spectrumcomponent(const Spectrumcomponent& spectrumComponent) :
        id(cmzn_spectrumcomponent_access(spectrumComponent.id))
    {
    }

    Spectrumcomponent& operator=(const Spectrumcomponent& spectrumComponent)
    {
        cmzn_spectrumcomponent_id temp_id = cmzn_spectrumcomponent_access(
            spectrumComponent.id);
        if (0 != id)
        {
            cmzn_spectrumcomponent_destroy(&id);
        }
        id = temp_id;
        return *this;
    }

    ~Spectrumcomponent()
    {
        if (0 != id)
        {
            cmzn_spectrumcomponent_destroy(&id);
        }
    }

    bool isValid() const
    {
        return (0 != id);
    }

    enum ScaleType
    {
        SCALE_TYPE_INVALID = CMZN_SPECTRUMCOMPONENT_SCALE_TYPE_INVALID,
        SCALE_TYPE_LINEAR = CMZN_SPECTRUMCOMPONENT_SCALE_TYPE_LINEAR,
        /*!< default ScaleType */
        SCALE_TYPE_LOG = CMZN_SPECTRUMCOMPONENT_SCALE_TYPE_LOG
    };

    enum ColourMappingType
    {
        COLOUR_MAPPING_TYPE_INVALID = CMZN_SPECTRUMCOMPONENT_COLOUR_MAPPING_TYPE_INVALID,
        COLOUR_MAPPING_TYPE_ALPHA = CMZN_SPECTRUMCOMPONENT_COLOUR_MAPPING_TYPE_ALPHA,
        COLOUR_MAPPING_TYPE_BANDED = CMZN_SPECTRUMCOMPONENT_COLOUR_MAPPING_TYPE_BANDED,
        COLOUR_MAPPING_TYPE_BLUE = CMZN_SPECTRUMCOMPONENT_COLOUR_MAPPING_TYPE_BLUE,
        COLOUR_MAPPING_TYPE_GREEN = CMZN_SPECTRUMCOMPONENT_COLOUR_MAPPING_TYPE_GREEN,
        COLOUR_MAPPING_TYPE_MONOCHROME = CMZN_SPECTRUMCOMPONENT_COLOUR_MAPPING_TYPE_MONOCHROME,
        COLOUR_MAPPING_TYPE_RAINBOW = CMZN_SPECTRUMCOMPONENT_COLOUR_MAPPING_TYPE_RAINBOW,
        /*!< default ColourMappingType */
        COLOUR_MAPPING_TYPE_RED = CMZN_SPECTRUMCOMPONENT_COLOUR_MAPPING_TYPE_RED,
        COLOUR_MAPPING_TYPE_STEP = CMZN_SPECTRUMCOMPONENT_COLOUR_MAPPING_TYPE_STEP,
        COLOUR_MAPPING_TYPE_WHITE_TO_BLUE = CMZN_SPECTRUMCOMPONENT_COLOUR_MAPPING_TYPE_WHITE_TO_BLUE,
        COLOUR_MAPPING_TYPE_WHITE_TO_RED = CMZN_SPECTRUMCOMPONENT_COLOUR_MAPPING_TYPE_WHITE_TO_RED,
        COLOUR_MAPPING_TYPE_WHITE_TO_GREEN = CMZN_SPECTRUMCOMPONENT_COLOUR_MAPPING_TYPE_WHITE_TO_GREEN
    };

    cmzn_spectrumcomponent_id getId() const
    {
        return id;
    }

    double getRangeMinimum()
    {
        return cmzn_spectrumcomponent_get_range_minimum(id);
    }

    int setRangeMinimum(double value)
    {
        return cmzn_spectrumcomponent_set_range_minimum(id, value);
    }

    double getRangeMaximum()
    {
        return cmzn_spectrumcomponent_get_range_maximum(id);
    }

    int setRangeMaximum(double value)
    {
        return cmzn_spectrumcomponent_set_range_maximum(id, value);
    }

    double getColourMinimum()
    {
        return cmzn_spectrumcomponent_get_colour_minimum(id);
    }

    int setColourMinimum(double value)
    {
        return cmzn_spectrumcomponent_set_colour_minimum(id,value);
    }

    double getColourMaximum()
    {
        return cmzn_spectrumcomponent_get_colour_maximum(id);
    }

    int setColourMaximum(double value)
    {
        return cmzn_spectrumcomponent_set_colour_maximum(id, value);
    }

    double getStepValue()
    {
        return cmzn_spectrumcomponent_get_step_value(id);
    }

    int setStepValue(double value)
    {
        return cmzn_spectrumcomponent_set_step_value(id, value);
    }

    double getExaggeration()
    {
        return cmzn_spectrumcomponent_get_exaggeration(id);
    }

    int setExaggeration(double value)
    {
        return cmzn_spectrumcomponent_set_exaggeration(id, value);
    }

    double getBandedRatio()
    {
        return cmzn_spectrumcomponent_get_banded_ratio(id);
    }

    int setBandedRatio(double value)
    {
        return cmzn_spectrumcomponent_set_banded_ratio(id, value);
    }

    bool isActive()
    {
        return cmzn_spectrumcomponent_is_active(id);
    }

    int setActive(bool active)
    {
        return cmzn_spectrumcomponent_set_active(id, active);
    }

    bool isColourReverse()
    {
        return cmzn_spectrumcomponent_is_colour_reverse(id);
    }

    int setColourReverse(bool reverse)
    {
        return cmzn_spectrumcomponent_set_colour_reverse(id, reverse);
    }

    bool isExtendAbove()
    {
        return cmzn_spectrumcomponent_is_extend_above(id);
    }

    int setExtendAbove(bool extend_above)
    {
        return cmzn_spectrumcomponent_set_extend_above(id, extend_above);
    }

    bool isExtendBelow()
    {
        return cmzn_spectrumcomponent_is_extend_below(id);
    }

    int setExtendBelow(bool extend_below)
    {
        return cmzn_spectrumcomponent_set_extend_below(id, extend_below);
    }

    int getFieldComponent()
    {
        return cmzn_spectrumcomponent_get_field_component(id);
    }

    int setFieldComponent(int componentNumber)
    {
        return cmzn_spectrumcomponent_set_field_component(id,
            componentNumber);
    }

    int getNumberOfBands()
    {
        return cmzn_spectrumcomponent_get_number_of_bands(id);
    }

    int setNumberOfBands(int numberOfBands)
    {
        return cmzn_spectrumcomponent_set_number_of_bands(id, numberOfBands);
    }

    ScaleType getScaleType()
    {
        return static_cast<ScaleType>(cmzn_spectrumcomponent_get_scale_type(
            id));
    }

    int setScaleType(ScaleType scaleType)
    {
        return cmzn_spectrumcomponent_set_scale_type(id,
            static_cast<cmzn_spectrumcomponent_scale_type>(scaleType));
    }

    ColourMappingType getColourMappingType()
    {
        return static_cast<ColourMappingType>(cmzn_spectrumcomponent_get_colour_mapping_type(
            id));
    }

    int setColourMappingType(ColourMappingType colourMapping)
    {
        return cmzn_spectrumcomponent_set_colour_mapping_type(id,
            static_cast<cmzn_spectrumcomponent_colour_mapping_type>(colourMapping));
    }

};

class Spectrum
{
protected:
    cmzn_spectrum_id id;

public:

    Spectrum() :
        id(0)
    {
    }

    // takes ownership of C handle, responsibility for destroying it
    explicit Spectrum(cmzn_spectrum_id in_spectrum_id) :
        id(in_spectrum_id)
    {
    }

    Spectrum(const Spectrum& spectrum) :
        id(cmzn_spectrum_access(spectrum.id))
    {
    }

    Spectrum& operator=(const Spectrum& spectrum)
    {
        cmzn_spectrum_id temp_id = cmzn_spectrum_access(spectrum.id);
        if (0 != id)
        {
            cmzn_spectrum_destroy(&id);
        }
        id = temp_id;
        return *this;
    }

    ~Spectrum()
    {
        if (0 != id)
        {
            cmzn_spectrum_destroy(&id);
        }
    }

    bool isValid() const
    {
        return (0 != id);
    }

    cmzn_spectrum_id getId() const
    {
        return id;
    }

    bool isManaged()
    {
        return cmzn_spectrum_is_managed(id);
    }

    int setManaged(bool value)
    {
        return cmzn_spectrum_set_managed(id, value);
    }

    int beginChange()
    {
        return cmzn_spectrum_begin_change(id);
    }

    int endChange()
    {
        return cmzn_spectrum_end_change(id);
    }

    char *getName()
    {
        return cmzn_spectrum_get_name(id);
    }

    int setName(const char *name)
    {
        return cmzn_spectrum_set_name(id, name);
    }

    int getNumberOfSpectrumcomponents()
    {
        return cmzn_spectrum_get_number_of_spectrumcomponents(id);
    }

    Spectrumcomponent createSpectrumcomponent()
    {
        return Spectrumcomponent(cmzn_spectrum_create_spectrumcomponent(id));
    }

    Spectrumcomponent getFirstSpectrumcomponent()
    {
        return Spectrumcomponent(cmzn_spectrum_get_first_spectrumcomponent(id));
    }

    Spectrumcomponent getNextSpectrumcomponent(const Spectrumcomponent& refComponent)
    {
        return Spectrumcomponent(
            cmzn_spectrum_get_next_spectrumcomponent(id, refComponent.getId()));
    }

    Spectrumcomponent getPreviousSpectrumcomponent(const Spectrumcomponent& refComponent)
    {
        return Spectrumcomponent(
            cmzn_spectrum_get_previous_spectrumcomponent(id, refComponent.getId()));
    }

    int moveSpectrumcomponentBefore(const Spectrumcomponent& component,
        const Spectrumcomponent& refComponent)
    {
        return cmzn_spectrum_move_spectrumcomponent_before(id, component.getId(),
            refComponent.getId());
    }

    int removeSpectrumcomponent(const Spectrumcomponent& component)
    {
        return cmzn_spectrum_remove_spectrumcomponent(id, component.getId());
    }

    int removeAllSpectrumcomponents()
    {
        return cmzn_spectrum_remove_all_spectrumcomponents(id);
    }

    bool isMaterialOverwrite()
    {
        return cmzn_spectrum_is_material_overwrite(id);
    }

    int setMaterialOverwrite(bool overwrite)
    {
        return cmzn_spectrum_set_material_overwrite(id, overwrite);
    }

};

inline bool operator==(const Spectrum& a, const Spectrum& b)
{
    return a.getId() == b.getId();
}

class Spectrumiterator
{
private:

    cmzn_spectrumiterator_id id;

public:

    Spectrumiterator() : id(0)
    { }

    // takes ownership of C handle, responsibility for destroying it
    explicit Spectrumiterator(cmzn_spectrumiterator_id iterator_id) :
        id(iterator_id)
    { }

    Spectrumiterator(const Spectrumiterator& spectrumiterator) :
        id(cmzn_spectrumiterator_access(spectrumiterator.id))
    { }

    Spectrumiterator& operator=(const Spectrumiterator& spectrumiterator)
    {
        cmzn_spectrumiterator_id temp_id = cmzn_spectrumiterator_access(spectrumiterator.id);
        if (0 != id)
        {
            cmzn_spectrumiterator_destroy(&id);
        }
        id = temp_id;
        return *this;
    }

    ~Spectrumiterator()
    {
        if (0 != id)
        {
            cmzn_spectrumiterator_destroy(&id);
        }
    }

    bool isValid() const
    {
        return (0 != id);
    }

    Spectrum next()
    {
        return Spectrum(cmzn_spectrumiterator_next(id));
    }
};

class Spectrummodule
{
protected:
    cmzn_spectrummodule_id id;

public:

    Spectrummodule() :
        id(0)
    {
    }

    // takes ownership of C handle, responsibility for destroying it
    explicit Spectrummodule(cmzn_spectrummodule_id in_spectrummodule_id) :
        id(in_spectrummodule_id)
    {
    }

    Spectrummodule(const Spectrummodule& spectrummodule) :
        id(cmzn_spectrummodule_access(spectrummodule.id))
    {
    }

    Spectrummodule& operator=(const Spectrummodule& spectrummodule)
    {
        cmzn_spectrummodule_id temp_id = cmzn_spectrummodule_access(
            spectrummodule.id);
        if (0 != id)
        {
            cmzn_spectrummodule_destroy(&id);
        }
        id = temp_id;
        return *this;
    }

    ~Spectrummodule()
    {
        if (0 != id)
        {
            cmzn_spectrummodule_destroy(&id);
        }
    }

    bool isValid() const
    {
        return (0 != id);
    }

    cmzn_spectrummodule_id getId() const
    {
        return id;
    }

    Spectrum createSpectrum()
    {
        return Spectrum(cmzn_spectrummodule_create_spectrum(id));
    }

    Spectrumiterator createSpectrumiterator()
    {
        return Spectrumiterator(cmzn_spectrummodule_create_spectrumiterator(id));
    }

    Spectrum findSpectrumByName(const char *name)
    {
        return Spectrum(cmzn_spectrummodule_find_spectrum_by_name(id, name));
    }

    int beginChange()
    {
        return cmzn_spectrummodule_begin_change(id);
    }

    int endChange()
    {
        return cmzn_spectrummodule_end_change(id);
    }

    Spectrum getDefaultSpectrum()
    {
        return Spectrum(cmzn_spectrummodule_get_default_spectrum(id));
    }

    int setDefaultSpectrum(const Spectrum& spectrum)
    {
        return cmzn_spectrummodule_set_default_spectrum(id, spectrum.getId());
    }
};

inline Spectrummodule Context::getSpectrummodule()
{
    return Spectrummodule(cmzn_context_get_spectrummodule(id));
}

} // namespace Zinc
}

#endif
